-- if player class is not hunter do not load anything
if (select(2, UnitClass("player"))) ~= "HUNTER" then return end

local L = AceLibrary("AceLocale-2.2"):new("SimpleMD")
local PlayerName = UnitName("player")
-- to be used soon because the hunters in my guild are lazy and I forsee version checks being needed
local revision = "$Revision: 45006 $"

local opts = {
	type='group',
	args = {
		msg1 = {
			type = 'text',
			name = L["Gain Message"],
			desc = L["Set Misdirection Gained Message"],
			usage = L["<your message>"],
			get = "GetMessageGain",
			set = "SetMessageGain",
        },
		msg2 = {
			type = 'text',
			name = L["Faded Message"],
			desc = L["Set Misdirection Faded Message"],
			usage = L["<your message>"],
			get = "GetMessageFade",
			set = "SetMessageFade",
		},
		color = {
			type = 'group',
			name = L["Text Color"],
			desc = L["Color for the On Screen Display"],
			args = {
				red = {
					type = 'range',
					name = L["Red Percent"],
					desc = L["Sets how much red is in the message text"],
					step = 0.1,
					min = 0.0,
					max = 1.0,
					isPercent = false,
					get = "getRed",
					set = "setRed",
				},
				green = {
					type = 'range',
					name = L["Green Percent"],
					desc = L["Sets how much green is in the message text"],
					step = 0.1,
					min = 0.0,
					max = 1.0,
					isPercent = false,
					get = "getGreen",
					set = "setGreen",
				},
				blue = {
					type = 'range',
					name = L["Blue Percent"],
					desc = L["Sets how much blue is in the message text"],
					step = 0.1,
					min = 0.0,
					max = 1.0,
					isPercent = false,
					get = "getBlue",
					set = "setBlue",
				},
			},
		},
		broadcast = {
			type = 'group',
			name = L["Broadcast Options"],
			desc = L["Options for how and where the alerts are displayed"],
			args = {
				screen = {
					type = 'toggle',
					name = L["Show on Screen"],
					desc = L["Toggles the display of the alert message on screen"],
					get = "IsShowOnScreen",
					set = "ToggleShowOnScreen",
				},
				console = {
					type = 'toggle',
					name = L["Show in Chat"],
					desc = L["Toggles the display of the alerts as self messages"],
					get = "IsShowInChat",
					set = "ToggleShowInChat",
				},
				channel = {
					type = 'toggle',
					name = L["Show in Channel"],
					desc = L["Toggles broadcasting of alerts to a user defined channel"],
					get = "IsShowInChannel",
					set = "ToggleShowInChannel",
				},
				floating = {
					type = 'toggle',
					name = L["Show in Combat Text"],
					desc = L["Toggles showing alerts in popular SCT like mods"],
					get = "IsShowFloating",
					set = "ToggleShowFloating",
				},
				showSelf = {
					type = 'toggle',
					name = L["Show Own Alerts"],
					desc = L["Toggles showing your alerts on your client"],
					get = "IsShowSelf",
					set = "ToggleShowSelf",
				},
				channel = {
					type = 'text',
					name = L["Custom Channel"],
					desc = L["Name of custom channel"],
					usage = L["<channel name>"],
					get = "GetChannel",
					set = "SetChannel",
				},
			},
		},
		cooldowns = {
			type = 'group',
			name = L["Cooldown Broadcast Options"],
			desc = L["Toggle which cooldown warnings you want to display to other users"],
			args = {
				ten = {
					type = 'toggle',
					name = L["10 Second Warning"],
					desc = L["Display a warning with 10 seconds left on MD cooldown"],
					get = "IsTenSeconds",
					set = "ToggleTenSeconds",
				},
				thirty = {
					type = 'toggle',
					name = L["30 Second Warning"],
					desc = L["Display a warning with 30 seconds left on MD cooldown"],
					get = "IsThirtySeconds",
					set = "ToggleThirtySeconds",
				},
				sixty = {
					type = 'toggle',
					name = L["60 Second Warning"],
					desc = L["Display a warning with 60 seconds left on MD cooldown"],
					get = "IsSixtySeconds",
					set = "ToggleSixtySeconds",
				},
			},
		},
		test = {
			type = 'execute',
			name = L["Test Message Settings"],
			desc = L["Prints a test message(s) to the screen, chat frame, and/or channel"],
			func = "messageTest",
		},
	},
}

SimpleMD = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceComm-2.0", "Sink-1.0")
SimpleMD:RegisterChatCommand(L["Slash-Commands"], opts)
SimpleMD:SetCommPrefix("SimpleMD")

SimpleMD:RegisterDB("SimplemdDB", "SimplemdDBPC")
SimpleMD:RegisterDefaults("profile", {
	messageGain = L["cast Misdirection on"],
	messageFade = L["Misdirection fades"],
	Red = 1.0,
	Green = 1.0,
	Blue = 1.0,
	channel = "",
	showInChannel = false,
	showInChat = false,
	showOnScreen = true,
	showFloating = false,
	showSelf = true,
	ten = true,
	thirty = true,
	sixty = false,
})

function SimpleMD:OnEnable()
	self:RegisterComm("SimpleMD", "GROUP", "OnCommReceive")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD1")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD2")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD3")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD4")
end

function SimpleMD:OnCommReceive(prefix, sender, distribution, message)
	--abbreviate the colors in local values so AddMessage() is readable
	local r = self.db.profile.Red
	local g = self.db.profile.Green
	local b = self.db.profile.Blue
	
	if self.db.profile.showInChat then
		self:Print(message)
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(message, r, g, b, 5.0)
	end
end

function SimpleMD:SendMessage(message)
	local channel = self.db.profile.channel
	
	SimpleMD:SendCommMessage("GROUP", message)
	
	if self.db.profile.showInChannel and self.db.profile.channel ~= "" then
		if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
			SendChatMessage(message, "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
		end
	end
	
	if self.db.profile.showFloating then
		self:Pour(message, 1, 1, 1)
	end
end

--check to see if spell is Misdirection, if it is set the spell target
function SimpleMD:UNIT_SPELLCAST_SENT(arg1, arg2, arg3, arg4)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self.SpellTarget = arg4
		end
	end
end

-- if spellcast succeeds call the gain function and schedule cooldown events
-- needed to make sure messages are not sent when target is out of LOS or out of range or invalid
function SimpleMD:UNIT_SPELLCAST_SUCCEEDED(arg1, arg2, arg3)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self:Gain()
			if self.db.profile.sixty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD1", 60) --60 second warning
			end
			if self.db.profile.thirty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD2", 90) --30 second warning
			end
			if self.db.profile.ten then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD3", 110) -- 10 second warning
			end
			self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD4", 120) -- cooldown finished
		end
	end
	
	--[[account for the readiness talent
	if arg1 == "player" and arg2 == L["Readiness"] then
		self:CancelAllScheduledEvents()
		local temp = PlayerName .. " " .. L["used Readiness! MD ready!"]
		self:SendMessage(temp)
	end]]
end

function SimpleMD:CHAT_MSG_SPELL_AURA_GONE_SELF(arg1)
	if arg1 == L["Misdirection fades from you."] then
		self:Fade()
	end
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD1()
	local temp = PlayerName .. " " .. L["has 60 seconds on MD cooldown."]
	self:SendMessage(temp)
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD2()
	local temp = PlayerName .. " " .. L["has 30 seconds on MD cooldown."]
	self:SendMessage(temp)
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD3()
	local temp = PlayerName .. " " .. L["has 10 seconds on MD cooldown!"]
	self:SendMessage(temp)
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD4()
	local temp = PlayerName .. " " .. L["has MD ready!"]
	self:SendMessage(temp)
end

function SimpleMD:Gain()
	--abbreviate the colors in local values so AddMessage() is readable
	local r = self.db.profile.Red
	local g = self.db.profile.Green
	local b = self.db.profile.Blue

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat then
			self:Print("You " .. self.db.profile.messageGain)
		end
		if self.db.profile.showOnScreen then
			UIErrorsFrame:AddMessage(L["You"] .. " " .. self.db.profile.messageGain .. " " .. self.SpellTarget, r, g, b, 5.0)
		end
	
		--local holder for AceComm message
		--used due to problem concatenation strings in parameter list
		local temp = PlayerName .. " " .. self.db.profile.messageGain .. " " .. self.SpellTarget
		self:SendMessage(temp)
	end
end

-- called on Misdirection fading from self
function SimpleMD:Fade()
	--abbreviate the colors in local values so AddMessage() is readable
	local r = self.db.profile.Red
	local g = self.db.profile.Green
	local b = self.db.profile.Blue

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat then
			self:Print(self.db.profile.messageFade)
		end
		if self.db.profile.showOnScreen then
			UIErrorsFrame:AddMessage(self.db.profile.messageFade, r, g, b, 5.0)
		end
		
		--local holder for AceComm message
		--used due to problem concatenation strings in parameter list
		local temp = self.db.profile.messageFade .. " " .. L["from"] .. " " .. self.SpellTarget
		self:SendMessage(temp)
	end
end

-- test broadcast for making sure colors or channel is correct
function SimpleMD:messageTest()
	if self.db.profile.showInChat then
		self:Print(L["Testing Chat Frame Printing"])
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(L["Testing On Screen Printing"], r, g, b, 5.0)
	end
	if self.db.profile.showInChannel and channel ~= "" then
		SendChatMessage(L["Testing Channel Printing"], "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
	end
	if self.db.profile.showFloating then
		self:Pour(L["Testing Floating Message"], 1, 1, 1)
	end
end

-- The many functions for the options menu
-- All the usual get/set pairs
function SimpleMD:GetMessageGain()
	return self.db.profile.messageGain
end

function SimpleMD:SetMessageGain(newMessage)
	self.db.profile.messageGain = newMessage
end

function SimpleMD:GetMessageFade()
	return self.db.profile.messageFade
end

function SimpleMD:SetMessageFade(newMessage)
	self.db.profile.messageFade = newMessage
end

function SimpleMD:IsShowInChat()
	return self.db.profile.showInChat
end

function SimpleMD:ToggleShowInChat()
	self.db.profile.showInChat = not self.db.profile.showInChat
end

function SimpleMD:IsShowOnScreen()
	return self.db.profile.showOnScreen
end

function SimpleMD:ToggleShowOnScreen()
	self.db.profile.showOnScreen = not self.db.profile.showOnScreen
end

function SimpleMD:getRed()
	return self.db.profile.Red
end

function SimpleMD:setRed(new)
	self.db.profile.Red = new
end

function SimpleMD:getGreen()
	return self.db.profile.Green
end

function SimpleMD:setGreen(new)
	self.db.profile.Green = new
end

function SimpleMD:getBlue()
	return self.db.profile.Blue
end

function SimpleMD:setBlue(new)
	self.db.profile.Blue = new
end

function SimpleMD:IsShowInChannel()
	return self.db.profile.showInChannel
end

function SimpleMD:ToggleShowInChannel()
	self.db.profile.showInChannel = not self.db.profile.showInChannel
end

function SimpleMD:GetChannel()
	return self.db.profile.channel
end

function SimpleMD:SetChannel(new)
	self.db.profile.channel = new
end

function SimpleMD:IsTenSeconds()
	return self.db.profile.ten
end

function SimpleMD:ToggleTenSeconds()
	self.db.profile.ten = not self.db.profile.ten
end

function SimpleMD:IsThirtySeconds()
	return self.db.profile.thirty
end

function SimpleMD:ToggleThirtySeconds()
	self.db.profile.thirty = not self.db.profile.thirty
end

function SimpleMD:IsSixtySeconds()
	return self.db.profile.sixty
end

function SimpleMD:ToggleSixtySeconds()
	self.db.profile.sixty = not self.db.profile.sixty
end

function SimpleMD:IsShowFloating()
	return self.db.profile.showFloating
end

function SimpleMD:ToggleShowFloating()
	self.db.profile.showFloating = not self.db.profile.showFloating
end

function SimpleMD:IsShowSelf()
	return self.db.profile.showSelf
end

function SimpleMD:ToggleShowSelf()
	self.db.profile.showSelf = not self.db.profile.showSelf
end
